#!/usr/bin/python
# textgrid2csv-offline01.py
# D. Gibbon
#  2015-03-15 (V02, includes filename in CSV records)
# Source: Gibbon, Dafydd and Jue Yu. Time Group Analyzer: Methodology And Implementation, The Phonetician 111/112:9-34. 2015.
#  Download on: http://www.isphs.org/Phonetician/Phonetician_111-112.pdf
# Adapted and ported to Python 3 by Maarten Renckens for READSEARCH at Hasselt University, Hasselt, Belgium to include first and second level subdirectories (31 dec 2016)
#  READSEARCH.be

#How to use this script?
#First, make sure Python 2.7 or Python 3.0 is installed on your device (https://www.python.org/)
#Then, open the commandprompt and change the directory to where all files are stored.
# An example on Windows: cd C:\Users\USERFOLDER\Desktop\FOLDER_WITH_FILES
# An example on Ubuntu 14: cd /home/USERFOLDER/Documents/FOLDER_WITH_FILES
# An example on Mac: cd ~/Desktop/FOLDER_WITH_FILES
#At last, cal this script by first calling Python and then the location of the script
# An example on Windows with python 2: C:\Python27\python.exe C:\Users\famil\Desktop\Python\textgrid-all2csv_with_subdirectories.py
# An example on Windows with python 3: py C:\Users\famil\Desktop\Python\textgrid-all2csv_with_subdirectories.py
# An example on Ubuntu 14 with python 2: python textgrid-all2csv_with_subdirectories.py
# An example on Ubuntu 14 with python 3: python3 textgrid-all2csv_with_subdirectories.py
# An example on Mac: python textgrid-all2csv_with_subdirectories.py
#On some occasions, it is possible that Windows Explorer should be refreshed (ctrl+r)

#-------------------------------------------------
import os, sys, re
# Define a subdirectory in which the combined files should be placed.
subdirectoryExportAll = "_CSV_COMBINED_EXPORT"
# Define CSV header for individual files.
header = '# ' + sys.argv[0] + 'TextGrid to CSV (D. Gibbon, 2008-11-23)\n# Open the file with OpenOffice.org Calc or MS-Excel.\nName\tTierType\tTierName\tLabel\tStart\tEnd\tDuration\n'
#Remember statistics
amountOfSubFolders = 0
amountOfSkippedFiles = 0

#-------------------------------------------------
# Text file input / output.
# extra info: http://stackoverflow.com/a/32973383

def inputtextlines(filename):
    try:
        handle = open(filename,'r')
        linelist = handle.readlines()
        handle.close()
        return linelist
    except:
        print('Could not input text')

def outputtext(filename, text):
    try:
        handle = open(filename,'w')
        handle.write(text)
        handle.close()
    except:
        print('Could not output text')

#-------------------------------------------------
# Conversion routine.

def converttextgrid2csv(textgridlines,textgridname):
    csvtext = ''
    for line in textgridlines[9:]:
        line = re.sub('\n','',line)
        line = re.sub('\n','',line)
        line = re.sub('^ *','',line)
        line = re.sub(' = ',';',line)
        linepair = line.split(';')
        if len(linepair) == 2:
            if linepair[0] == 'class':
                classname = linepair[1]
            if linepair[0] == 'name':
                tiername = linepair[1]
            if linepair[0] == 'xmin':
                xmin = linepair[1]
            if linepair[0] == 'xmax':
                xmax = linepair[1]
            if linepair[0] == 'text':
                text = linepair[1]
                diff = str(float(xmax)-float(xmin))
                csvtext += textgridname + '\t' + classname + '\t' + tiername + '\t' + text + '\t' + xmin + '\t' + xmax + '\t' + diff + '\n'
    return csvtext

#-------------------------------------------------
# Process one TextGrid file at a time.

def convertTextgrid(textgridfiles, subdir):
    try:
        countConvertedFiles = 0
        allcsvs = ''
        for filename in textgridfiles:
            adjustedFilename = os.path.join(subdir, filename)

    # Create TextGrid name and CSV file name.
            tgname = re.sub('.TextGrid','',adjustedFilename)
            csvname = re.sub('.TextGrid','.csv',adjustedFilename)
    
    # Conversion.
            countConvertedFiles += 1
            try:
                textgrid = inputtextlines(adjustedFilename)
                csvtext = converttextgrid2csv(textgrid,tgname)
                if csvtext == '':
                    print('No data in file', adjustedFilename)
            except:
                print("Problem with TextGrid data.")
    
    # Create separate CSV output for each TextGrid.
            csvheadedtext = header + csvtext
            outputtext(csvname,csvheadedtext)
    
    # Put all CSVs into a single file.
            allcsvs += csvtext
    
        #Indicate from which folder the combination is extracted:
        outputtext(os.path.join(subdirectoryExportAll, (subdir.replace("\\", "_").replace("/", "_").replace(".", "_") + '_-_ALLCSVs.csv') ),allcsvs)
        countConvertedFiles2 = str(countConvertedFiles).zfill(4)
        print('%s TextGrids to csv in directory "%s" and added a resume in %s' % (countConvertedFiles2, subdir, subdirectoryExportAll))
    
    except:
        print("Unknown error.")

#-------------------------------------------------
#-------------------------------------------------
#-------------------------------------------------
# Prepare a folder for the combined export.

try:
	if not os.path.exists(subdirectoryExportAll):
		os.mkdir(subdirectoryExportAll)
		print('created a subdirectory for the combined export.')
except:
	print("Could not create a subdirectory for the combined export.")

#-------------------------------------------------
# Get starting filenames.
try:
    allFiles = sorted(os.listdir('.'))
except:
    print("Problem with inputting files.")

#-------------------------------------------------
# Check for TextGrid filenames.
def extractTextgridFiles(givenFiles, subdir):
    global amountOfSkippedFiles
    textgridfiles = [x for x in givenFiles if x.endswith(('.TextGrid'))]
    for y in textgridfiles:
        if y.startswith(('._')):
            textgridfiles.remove(y)
            amountOfSkippedFiles += 1
    if textgridfiles == []:
        print("No TextGrid files to process in the folder %s ." % (subdir))
    else:
        convertTextgrid(textgridfiles, subdir)

try:
    #first, process the textgrids found in this directory.
    extractTextgridFiles(allFiles, '.')
    #afterwards, process all first-level subdirectories.
    foundDirectories1 = [x for x in allFiles if os.path.isdir(x)]
    for x in foundDirectories1:
        if x == subdirectoryExportAll:
            print('skipping the folder %s' % (subdirectoryExportAll))
        if x != subdirectoryExportAll:
            amountOfSubFolders += 1
            try:
                #get all filenames for each subdirectory again.
                allFiles = sorted(os.listdir(os.path.join('.', x)))
                #process the textgrids found in this directory.
                extractTextgridFiles(allFiles, os.path.join('.', x))
                #process all second-level subdirectories
                foundDirectories2 = [y for y in allFiles if os.path.isdir(os.path.join('.', x, y))]
                for y in foundDirectories2:
                    amountOfSubFolders += 1
                    allFiles = sorted(os.listdir(os.path.join('.', x, y)))
                    extractTextgridFiles(allFiles, os.path.join('.', x, y))
            except:
                print("Problem within the subdirectories. I cannot continue.")

except:
    print("File input problem.")
    exit()

print("======================================================")
print("Processed %s subfolders in this directory" % (amountOfSubFolders) )
print("Skipped %s items starting with the filename ._"  % (amountOfSkippedFiles))
print("    (Probably invisible Mac files)")
print("Job finished =========================================")

#--- EOF -----------------------------------------
